package com.tencent.cloud.rn.callkit.utils;

import com.facebook.react.bridge.ReadableMap;
import com.tencent.qcloud.tuikit.TUICommonDefine;
import com.tencent.qcloud.tuikit.tuicallengine.TUICallDefine;

public class ParamsTransformer {

  public static TUICallDefine.MediaType transMediaType(Integer type) {
    switch (type) {
      case 0:
        return TUICallDefine.MediaType.Audio;
      default:
        return TUICallDefine.MediaType.Video;
    }
  }

  public static TUICallDefine.CallParams transCallParams(ReadableMap targetParams) {
    TUICallDefine.CallParams callParams = new TUICallDefine.CallParams();
    if (targetParams == null) {
      return callParams;
    }
    if (targetParams.hasKey(Constants.roomId) && !targetParams.isNull(Constants.roomId)) {
      callParams.roomId = transTUIRoomId(targetParams.getMap(Constants.roomId));
    }
    if (targetParams.hasKey(Constants.timeout) && !targetParams.isNull(Constants.timeout)) {
      callParams.timeout = targetParams.getInt(Constants.timeout);
    }
    if (targetParams.hasKey(Constants.userData) && !targetParams.isNull(Constants.userData)) {
      callParams.userData = targetParams.getString(Constants.userData);
    }
    if (targetParams.hasKey(Constants.offlinePushInfo) && !targetParams.isNull(Constants.offlinePushInfo)) {
      ReadableMap readableMap = targetParams.getMap(Constants.offlinePushInfo);
      callParams.offlinePushInfo = transOfflinePushInfo(readableMap);
    }
    return callParams;
  }

  public static TUICallDefine.OfflinePushInfo transOfflinePushInfo(ReadableMap params) {
    TUICallDefine.OfflinePushInfo OfflinePushInfo = new TUICallDefine.OfflinePushInfo();
    if (params == null) {
      return OfflinePushInfo;
    }
    if (params.hasKey(Constants.title) && !params.isNull(Constants.title)) {
      String title = params.getString(Constants.title);
      OfflinePushInfo.setTitle(title);
    }
    if (params.hasKey(Constants.isDisablePush) && !params.isNull(Constants.isDisablePush)) {
      boolean disablePush = params.getBoolean(Constants.isDisablePush);
      OfflinePushInfo.setDisablePush(disablePush);
    }
    if (params.hasKey(Constants.iOSPushType) && !params.isNull(Constants.iOSPushType)) {
      int type = params.getInt(Constants.iOSPushType);
      OfflinePushInfo.setIOSPushType(transIOSPushType(type));
    }
    if (params.hasKey(Constants.desc) && !params.isNull(Constants.desc)) {
      String desc = params.getString(Constants.desc);
      OfflinePushInfo.setDesc(desc);
    }
    if (params.hasKey(Constants.androidSound) && !params.isNull(Constants.androidSound)) {
      String androidSound = params.getString(Constants.androidSound);
      OfflinePushInfo.setAndroidSound(androidSound);
    }
    if (params.hasKey(Constants.androidFCMChannelID) && !params.isNull(Constants.androidFCMChannelID)) {
      String androidFCMChannelID = params.getString(Constants.androidFCMChannelID);
      OfflinePushInfo.setAndroidFCMChannelID(androidFCMChannelID);
    }
    if (params.hasKey(Constants.androidHuaWeiCategory) && !params.isNull(Constants.androidHuaWeiCategory)) {
      String androidHuaWeiCategory = params.getString(Constants.androidHuaWeiCategory);
      OfflinePushInfo.setAndroidHuaWeiCategory(androidHuaWeiCategory);
    }
    if (params.hasKey(Constants.androidOPPOChannelID) && !params.isNull(Constants.androidOPPOChannelID)) {
      String androidOPPOChannelID = params.getString(Constants.androidHuaWeiCategory);
      OfflinePushInfo.setAndroidOPPOChannelID(androidOPPOChannelID);
    }
    if (params.hasKey(Constants.androidXiaoMiChannelID) && !params.isNull(Constants.androidXiaoMiChannelID)) {
      String androidXiaoMiChannelID = params.getString(Constants.androidXiaoMiChannelID);
      OfflinePushInfo.setAndroidXiaoMiChannelID(androidXiaoMiChannelID);
    }
    if (params.hasKey(Constants.ignoreIOSBadge) && !params.isNull(Constants.ignoreIOSBadge)) {
      boolean ignoreIOSBadge = params.getBoolean(Constants.ignoreIOSBadge);
      OfflinePushInfo.setIgnoreIOSBadge(ignoreIOSBadge);
    }
    if (params.hasKey(Constants.iOSSound) && !params.isNull(Constants.iOSSound)) {
      String iOSSound = params.getString(Constants.iOSSound);
      OfflinePushInfo.setIOSSound(iOSSound);
    }
    if (params.hasKey(Constants.androidVIVOClassification) && !params.isNull(Constants.androidVIVOClassification)) {
      int androidVIVOClassification = params.getInt(Constants.androidVIVOClassification);
      OfflinePushInfo.setAndroidVIVOClassification(androidVIVOClassification);
    }
    return OfflinePushInfo;
  }

  public static TUICommonDefine.RoomId transTUIRoomId(ReadableMap targetRoomId) {
    TUICommonDefine.RoomId roomId = new TUICommonDefine.RoomId();
    if (targetRoomId == null) {
      return roomId;
    }
    if (targetRoomId.hasKey(Constants.intRoomId) && !targetRoomId.isNull(Constants.intRoomId)) {
      roomId.intRoomId = targetRoomId.getInt(Constants.intRoomId);
    }
    if (targetRoomId.hasKey(Constants.strRoomId) && !targetRoomId.isNull(Constants.strRoomId)) {
      roomId.strRoomId = targetRoomId.getString(Constants.strRoomId);
    }
    return roomId;
  }

  public static TUICallDefine.IOSOfflinePushType transIOSPushType(Integer type) {
    if (type.equals(0)) {
      return TUICallDefine.IOSOfflinePushType.APNs;
    }
    return TUICallDefine.IOSOfflinePushType.VoIP;
  }

  public static int transRoleToInt(TUICallDefine.Role role) {
    switch (role) {
      case None -> {
        return 0;
      }
      case Caller -> {
        return 1;
      }
      default -> {
        return 2;
      }
    }
  }

}
