import TUICallKit_Swift
import TUICallEngine
import Foundation

extension CallUikit: TUICallObserver {
  
  override func supportedEvents() -> [String]! {
    return ["onListener"]
  }
  
  // Will be called when this module's first listener is add.
  override func startObserving() {
    TUICallEngine.createInstance().addObserver(self)
  }

  // Will be called when this module's last listener is removed.
  override func stopObserving() {
    TUICallEngine.createInstance().removeObserver(self)
  }

  func onError(code: Int32, message: String?) {
    var param: [String: Any] = [:]
    param["code"] = code
    param["message"] = message
    dispatchEvent(eventName:"onError", param)
  }
  
  func onCallReceived(callerId: String, calleeIdList: [String], groupId: String?, callMediaType: TUICallMediaType) {
    var param: [String: Any] = [:]
    param["callerId"] = callerId
    param["calleeIdList"] = calleeIdList
    param["groupId"] = groupId
    param["callMediaType"] = callMediaType == TUICallMediaType.audio ? 0 : 1
    dispatchEvent(eventName:"onCallReceived", param)
  }
  
  func onCallCancelled(callerId: String) {
    var param: [String: Any] = [:]
    param["callerId"] = callerId
    dispatchEvent(eventName:"onCallCancelled", param)
  }
  
  func onCallBegin(roomId: TUIRoomId, callMediaType: TUICallMediaType, callRole: TUICallRole) {
    var param: [String: Any] = [:]
    var tuiRoomId:[String: Any] = [:]
    tuiRoomId["intRoomId"] = roomId.intRoomId
    tuiRoomId["strRoomId"] = roomId.strRoomId
    
    param["roomId"] = tuiRoomId
    param["callMediaType"] = callMediaType == TUICallMediaType.audio ? 0 : 1
    param["callRole"] = transTUICallRole(callRole: callRole)
    dispatchEvent(eventName:"onCallBegin", param)
  }
  
  func onCallEnd(roomId: TUIRoomId, callMediaType: TUICallMediaType, callRole: TUICallRole, totalTime: Float) {
    var param: [String: Any] = [:]
    var tuiRoomId:[String: Any] = [:]
    tuiRoomId["intRoomId"] = roomId.intRoomId
    tuiRoomId["strRoomId"] = roomId.strRoomId
    
    param["roomId"] = tuiRoomId
    param["callMediaType"] = callMediaType == TUICallMediaType.audio ? 0 : 1
    param["callRole"] = transTUICallRole(callRole: callRole)
    param["totalTime"] = totalTime
    dispatchEvent(eventName:"onCallEnd", param)
  }
  
  func onUserReject(userId: String) {
    var param: [String: Any] = [:]
    param["userId"] = userId
    dispatchEvent(eventName: "onUserReject", param)
  }
  
  func onUserNoResponse(userId: String) {
    var param: [String: Any] = [:]
    param["userId"] = userId
    dispatchEvent(eventName: "onUserNoResponse", param)
  }
  
  func onUserLineBusy(userId: String) {
    var param: [String: Any] = [:]
    param["userId"] = userId
    dispatchEvent(eventName: "onUserLineBusy", param)
  }
  
  func onUserJoin(userId: String) {
    var param: [String: Any] = [:]
    param["userId"] = userId
    dispatchEvent(eventName: "onUserJoin", param)
  }
  
  func onCallMediaTypeChanged(oldCallMediaType: TUICallMediaType, newCallMediaType: TUICallMediaType) {
    var param: [String: Any] = [:]
    param["oldCallMediaType"] = oldCallMediaType == TUICallMediaType.audio ? 0 : 1
    param["newCallMediaType"] = newCallMediaType == TUICallMediaType.audio ? 0 : 1
    dispatchEvent(eventName: "onCallMediaTypeChanged", param)
  }
  
  func onKickedOffline() {
    var param: [String: Any] = [:]
    dispatchEvent(eventName:"onKickedOffline", param)
  }
  
  func onUserSigExpired() {
    var param: [String: Any] = [:]
    dispatchEvent(eventName:"onUserSigExpired", param)
  }
  
  func dispatchEvent(eventName:String, _ object: Any) {
    if let jsonString = convertNSObjectToJsonString(object) {
      sendEvent(withName: "onListener", body: ["type": eventName, "params": jsonString])
    }
  }
  
  func convertNSObjectToJsonString(_ object: Any) -> String? {
      do {
          let jsonData = try JSONSerialization.data(withJSONObject: object, options: [])
          return String(data: jsonData, encoding: .utf8)
      } catch {
          return nil
      }
  }
  
  private func transTUICallRole(callRole: TUICallRole) -> Int {
    if (callRole == TUICallRole.call) {
      return 1
    }
    if (callRole == TUICallRole.called) {
      return 2
    }
    return 0
  }
  
}
