import TUICore
import TUICallKit_Swift
import TUICallEngine
import Foundation

@objc(CallUikit)
class CallUikit: RCTEventEmitter {
  
  @objc(login:userId:userSig:onSuccess:onError:)
  func login(sdkAppId: NSNumber, userId: NSString, userSig: NSString, onSuccess: @escaping RCTResponseSenderBlock, onError: @escaping RCTResponseSenderBlock) {
    DispatchQueue.main.async {
      TUILogin.login(sdkAppId.int32Value , userID: userId.description , userSig: userSig.description) { [weak self] in
        guard let self = self else { return }
        onSuccess(["Success."])
      } fail: { errorCode, errorMessage in
        onError([errorCode, errorMessage])
      }
    }
  }

  @objc(logout:onError:)
  func logout(onSuccess: @escaping RCTResponseSenderBlock, onError: @escaping RCTResponseSenderBlock) {
    DispatchQueue.main.async {
      TUILogin.logout {[weak self] in
        guard let self = self else { return }
        self.destroyInstance()
        onSuccess(["Success."])
      } fail: { errorCode, errorMessage in
        onError([errorCode, errorMessage])
      }
    }
  }

  @objc(setSelfInfo:avatar:onSuccess:onError:)
  func setSelfInfo(nickname: NSString, avatar: NSString, onSuccess: @escaping RCTResponseSenderBlock, onError: @escaping RCTResponseSenderBlock) {
    DispatchQueue.main.async {
      TUICallKit.createInstance().setSelfInfo(nickname: nickname.description, avatar: avatar.description) {
        onSuccess(["Success."])
      } fail: { code, errorMessage in
        onError([code, errorMessage])
      }
    }
  }

  @objc(call:callMediaType:params:onSuccess:onError:)
  func call(userId: NSString, callMediaType: Double, params: NSDictionary?, onSuccess: @escaping RCTResponseSenderBlock, onError: @escaping RCTResponseSenderBlock) {
    let mediaType: TUICallMediaType = transMediaType(callMediaType)
    let callParams: TUICallParams = transCallParams(params)
    DispatchQueue.main.async {
      TUICallKit.createInstance().call(userId: userId.description, callMediaType: mediaType, params: callParams) {
        onSuccess(["Success."])
      } fail: { code, errorMessage in
        onError([code, errorMessage])
      }
    }
  }
  
  @objc(groupCall:userIdList:callMediaType:params:onSuccess:onError:)
  func groupCall(groupId: NSString, userIdList: [NSString], callMediaType: Double, params: NSDictionary, onSuccess: @escaping RCTResponseSenderBlock, onError: @escaping RCTResponseSenderBlock) {
    let mediaType: TUICallMediaType = transMediaType(callMediaType)
    let callParams: TUICallParams = transCallParams(params)
    DispatchQueue.main.async {
      TUICallKit.createInstance().groupCall(groupId: groupId.description, userIdList: userIdList as [String], callMediaType: mediaType, params: callParams) {
        onSuccess(["Success."])
      } fail: { code, message in
        onError([code, message])
      }
    }
  }
  
  @objc(joinInGroupCall:groupId:callMediaType:)
  func joinInGroupCall(roomId:NSDictionary, groupId: NSString, callMediaType: Double) {
    let tuiRoomId: TUIRoomId = transTUIRoomId(roomId)
    let mediaType: TUICallMediaType = transMediaType(callMediaType)
    DispatchQueue.main.async {
      TUICallKit.createInstance().joinInGroupCall(roomId: tuiRoomId, groupId: groupId.description, callMediaType: mediaType)
    }
  }

  @objc(setCallingBell:)
  func setCallingBell(filePath: NSString) {
    DispatchQueue.main.async {
      TUICallKit.createInstance().setCallingBell(filePath: filePath.description)
    }
  }

  @objc(enableMuteMode:)
  func enableMuteMode(enable: Bool) {
    DispatchQueue.main.async {
      TUICallKit.createInstance().enableMuteMode(enable: enable)
    }
  }

  @objc(enableVirtualBackground:)
  func enableVirtualBackground(enable: Bool) {
    DispatchQueue.main.async {
      TUICallKit.createInstance().enableVirtualBackground(enable: enable)
    }
  }

  @objc(enableIncomingBanner:)
  func enableIncomingBanner(enable: Bool) {
    DispatchQueue.main.async {
      TUICallKit.createInstance().enableIncomingBanner(enable: enable)
    }
  }
  
  @objc(setScreenOrientation:)
  func setScreenOrientation(orientation: NSNumber) {
    
  }
    
  private func destroyInstance() {
    TUICallEngine.createInstance().removeObserver(self)
    TUICallEngine.destroyInstance();
  }
  
  private func transMediaType(_ type: Double) -> TUICallMediaType {
    switch type {
      case 0:
      return TUICallMediaType.audio
      default:
      return TUICallMediaType.video
    }
  }
  
  private func transCallParams(_ params: NSDictionary?) -> TUICallParams {
    var callParams: TUICallParams = TUICallParams()
    guard let params = params else { return callParams }
    if let roomId = params["roomId"] as? NSDictionary {
      callParams.roomId = transTUIRoomId(roomId);
    }
    if let timeout = params["timeout"] as? Int32 {
      callParams.timeout = timeout
    }
    if let userData = params["userData"] as? String {
      callParams.userData = userData
    }
    if let offlinePushInfo = params["offlinePushInfo"] as? NSDictionary {
      callParams.offlinePushInfo = transOfflinePushInfo(offlinePushInfo)
    }
    return callParams
  }
  
  private func transTUIRoomId(_ targetRoomId: NSDictionary?) -> TUIRoomId {
    var roomId: TUIRoomId = TUIRoomId();
    guard let targetRoomId = targetRoomId else { return roomId }
    if let intRoomId = targetRoomId["intRoomId"] as? UInt32 {
      roomId.intRoomId = intRoomId
    }
    if let strRoomId = targetRoomId["strRoomId"] as? NSString {
      roomId.strRoomId = strRoomId as String
    }
    return roomId
  }
  
  private func transOfflinePushInfo(_ params: NSDictionary?) -> TUIOfflinePushInfo {
    var offlinePushInfo: TUIOfflinePushInfo = TUIOfflinePushInfo()
    guard let params = params else { return offlinePushInfo }
    if let title = params["title"] as? String {
      offlinePushInfo.title = title
    }
    if let disablePush = params["isDisablePush"] as? Bool {
      offlinePushInfo.isDisablePush = disablePush
    }
    if let iOSPushType = params["iOSPushType"] as? Double {
      offlinePushInfo.iOSPushType = transIOSPushType(iOSPushType)
    }
    if let desc = params["desc"] as? String {
      offlinePushInfo.desc = desc
    }
    if let androidSound = params["androidSound"] as? String {
      offlinePushInfo.androidSound = androidSound
    }
    if let androidFCMChannelID = params["androidFCMChannelID"] as? String {
      offlinePushInfo.androidFCMChannelID = androidFCMChannelID
    }
    if let androidHuaWeiCategory = params["androidHuaWeiCategory"] as? String {
      offlinePushInfo.androidHuaWeiCategory = androidHuaWeiCategory
    }
    if let androidOPPOChannelID = params["androidOPPOChannelID"] as? String {
      offlinePushInfo.androidOPPOChannelID = androidOPPOChannelID
    }
    if let androidXiaoMiChannelID = params["androidXiaoMiChannelID"] as? String {
      offlinePushInfo.androidXiaoMiChannelID = androidXiaoMiChannelID
    }
    if let ignoreIOSBadge = params["ignoreIOSBadge"] as? Bool {
      offlinePushInfo.ignoreIOSBadge = ignoreIOSBadge
    }
    if let iOSSound = params["iOSSound"] as? String {
      offlinePushInfo.iOSSound = iOSSound
    }
    if let androidVIVOClassification = params["androidVIVOClassification"] as? Int {
      offlinePushInfo.androidVIVOClassification = androidVIVOClassification
    }
    return offlinePushInfo
  }
  
  private func transIOSPushType(_ type: Double) -> TUICallIOSOfflinePushType {
    if type == 0 {
      return TUICallIOSOfflinePushType.apns
    }
    return TUICallIOSOfflinePushType.voIP
  }
}
