"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _reactNative = require("react-native");
const CallUikit = _reactNative.NativeModules.CallUikit ? _reactNative.NativeModules.CallUikit : {};
const CallEventEmitter = new _reactNative.NativeEventEmitter(CallUikit);
class TUICallKit {
  constructor() {
    this._listeners = new Map();
  }
  static createInstance() {
    if (!TUICallKit._call) {
      TUICallKit._call = new TUICallKit();
    }
    return TUICallKit._call;
  }
  on(listener) {
    const callback = args => {
      let params;
      params = JSON.parse(args.params);
      listener(args.type, params);
    };
    CallEventEmitter.addListener('onListener', callback);
    this._listeners.set(listener, callback);
    return {
      remove: () => {
        this.off(listener);
      }
    };
  }
  off(listener) {
    const callback = this._listeners.get(listener);
    if (!callback) {
      return;
    }
    CallEventEmitter.removeAllListeners('onListener');
  }
  login(params, onSuccess, onError) {
    return CallUikit.login(params.sdkAppId, params.userId, params.userSig, onSuccess, onError);
  }
  logout(onSuccess, onError) {
    return CallUikit.logout(onSuccess, onError);
  }
  call(params, onSuccess, onError) {
    return CallUikit.call(params.userId, params.mediaType, params.callParams, onSuccess, onError);
  }
  groupCall(params, onSuccess, onError) {
    return CallUikit.groupCall(params.groupId, params.userIdList, params.mediaType, params.callParams, onSuccess, onError);
  }
  joinInGroupCall(params) {
    return CallUikit.joinInGroupCall(params.roomId, params.groupId, params.mediaType);
  }
  setCallingBell(filePath) {
    return CallUikit.setCallingBell(filePath);
  }
  enableMuteMode(enable) {
    return CallUikit.enableMuteMode(enable);
  }
  enableVirtualBackground(enable) {
    return CallUikit.enableVirtualBackground(enable);
  }
  setScreenOrientation(orientation) {
    return CallUikit.setScreenOrientation(orientation);
  }
}
exports.default = TUICallKit;
//# sourceMappingURL=callUIKit.js.map