import { NativeModules, NativeEventEmitter } from 'react-native';
import { TUICallEvent } from './callEvent';

import type {
  ILoginParams,
  ICallParams,
  IGroupCallParams,
  IJoinGroupCallParams,
} from './interface';

const CallUikit = NativeModules.CallUikit ? NativeModules.CallUikit : {};
const CallEventEmitter = new NativeEventEmitter(CallUikit);

export default class TUICallKit {
  static _call: TUICallKit | undefined;
  private _listeners: Map<any, any>;
  constructor() {
    this._listeners = new Map();
  }

  static createInstance() {
    if (!TUICallKit._call) {
      TUICallKit._call = new TUICallKit();
    }
    return TUICallKit._call;
  }

  on(listener: { (type: TUICallEvent, params: any): void }) {
    const callback = (args: { params: any; type: TUICallEvent }) => {
      let params;
      params = JSON.parse(args.params);
      listener(args.type, params);
    };
    CallEventEmitter.addListener('onListener', callback);
    this._listeners.set(listener, callback);
    return {
      remove: () => {
        this.off(listener);
      },
    };
  }

  off(listener: { (type: TUICallEvent, params: any): void }) {
    const callback = this._listeners.get(listener);
    if (!callback) {
      return;
    }
    CallEventEmitter.removeAllListeners('onListener');
  }

  login(
    params: ILoginParams,
    onSuccess?: (data: String) => void,
    onError?: (errCode: number, errMsg: string) => void
  ): void {
    return CallUikit.login(
      params.sdkAppId,
      params.userId,
      params.userSig,
      onSuccess,
      onError
    );
  }

  logout(
    onSuccess: (data: String) => void,
    onError?: (errCode: number, errMsg: string) => void
  ): void {
    return CallUikit.logout(onSuccess, onError);
  }

  call(
    params: ICallParams,
    onSuccess?: (data: String) => void,
    onError?: (errCode: number, errMsg: string) => void
  ): void {
    return CallUikit.call(
      params.userId,
      params.mediaType,
      params.callParams,
      onSuccess,
      onError
    );
  }

  groupCall(
    params: IGroupCallParams,
    onSuccess?: (data: String) => void,
    onError?: (errCode: number, errMsg: string) => void
  ): void {
    return CallUikit.groupCall(
      params.groupId,
      params.userIdList,
      params.mediaType,
      params.callParams,
      onSuccess,
      onError
    );
  }

  joinInGroupCall(params: IJoinGroupCallParams): void {
    return CallUikit.joinInGroupCall(
      params.roomId,
      params.groupId,
      params.mediaType
    );
  }

  setCallingBell(filePath: string): void {
    return CallUikit.setCallingBell(filePath);
  }

  enableMuteMode(enable: boolean): void {
    return CallUikit.enableMuteMode(enable);
  }

  enableVirtualBackground(enable: boolean): void {
    return CallUikit.enableVirtualBackground(enable);
  }

  setScreenOrientation(orientation: number): void {
    return CallUikit.setScreenOrientation(orientation);
  }
}
